@@ -6,8 +6,10 @@
 """Tests that can be imported and used to test optimizer implementations against this
 packages blackbox optimizer interface."""
 
-from typing import List, Optional, Type, Union
+import random
+from typing import Dict, List, Optional, Type, Union
 
+import numpy as np
 import parameterspace as ps
 
 from blackboxopt import Objective, OptimizationComplete, Optimizer
@@ -28,9 +30,9 @@ def _initialize_optimizer(
 ) -> Optimizer:
     if space is None:
         space = ps.ParameterSpace()
-        space.add(ps.IntegerParameter("p1", bounds=[1, 32], transformation="log"))
-        space.add(ps.ContinuousParameter("p2", [-2, 2]))
-        space.add(ps.ContinuousParameter("p3", [0, 1]))
+        space.add(ps.IntegerParameter("p1", bounds=(1, 32), transformation="log"))
+        space.add(ps.ContinuousParameter("p2", (-2, 2)))
+        space.add(ps.ContinuousParameter("p3", (0, 1)))
         space.add(ps.CategoricalParameter("p4", [True, False]))
         space.add(ps.OrdinalParameter("p5", ("small", "medium", "large")))
 
@@ -49,7 +51,7 @@ def optimize_single_parameter_sequentially_for_n_max_evaluations(
     ],
     optimizer_kwargs: dict,
     n_max_evaluations: int = 20,
-) -> bool:
+):
     """[summary]
 
     Args:
@@ -102,15 +104,13 @@ def optimize_single_parameter_sequentially_for_n_max_evaluations(
         evaluation = eval_spec.create_evaluation(objectives=evaluation_result)
         optimizer.report(evaluation)
 
-    return True
 
-
-def is_deterministic_with_fixed_seed(
+def is_deterministic_with_fixed_seed_and_larger_space(
     optimizer_class: Union[
         Type[SingleObjectiveOptimizer], Type[MultiObjectiveOptimizer]
     ],
     optimizer_kwargs: dict,
-) -> bool:
+):
     """Check if optimizer is deterministic.
 
     Repeatedly initialize the optimizer with the same parameter space and a fixed seed,
@@ -118,6 +118,9 @@ def is_deterministic_with_fixed_seed(
     evaluation specification. The configuration of all final evaluation specifications
     should be equal.
 
+
+    This tests covers multiple parameter types by using a mixed search space.
+
     Args:
         optimizer_class: Optimizer to test.
         optimizer_kwargs: Expected to contain additional arguments for initializing
@@ -145,7 +148,82 @@ def is_deterministic_with_fixed_seed(
         final_configurations.append(es2.configuration.copy())
 
     assert final_configurations[0] == final_configurations[1]
-    return True
+
+
+def is_deterministic_when_reporting_shuffled_evaluations(
+    optimizer_class: Union[
+        Type[SingleObjectiveOptimizer], Type[MultiObjectiveOptimizer]
+    ],
+    optimizer_kwargs: dict,
+):
+    """Check if determinism isn't affected by the order of initially reported data.
+
+    Repeatedly initialize the optimizer with the same parameter space and a fixed seed.
+    Report a set of initial evaluations in randomized order as initial data. Start
+    optimizing and check if the generated configurations for all optimizers are equal.
+
+    By doing multiple evaluations, this tests covers effects that become visible after
+    a while, e.g. only after stages got completed in staged iteration samplers.
+
+    Args:
+        optimizer_class: Optimizer to test.
+        optimizer_kwargs: Expected to contain additional arguments for initializing
+            the optimizer. (`search_space` and `objective(s)` are set automatically
+            by the test.)
+
+    Returns:
+        `True` if the test is passed.
+    """
+
+    space = ps.ParameterSpace()
+    space.add(ps.ContinuousParameter("p1", (0, 1)))
+
+    def _run_experiment_1d(es):
+        x = es.configuration["p1"]
+        _x = np.copy(np.atleast_2d(x))
+        params = np.array([0.75, 0.0, -10.0, 0.0, 0.0])
+        y = np.polyval(params, _x)
+        return float(np.squeeze(y))
+
+    runs: Dict[int, Dict] = {0: {}, 1: {}}
+    for run_idx, run in runs.items():
+        run["evaluations"] = []
+        opt = _initialize_optimizer(
+            optimizer_class,
+            optimizer_kwargs,
+            objective=Objective("loss", False),
+            objectives=[Objective("loss", False)],
+            space=space,
+            seed=0,
+        )
+
+        # Report initial data in different order
+        eval_specs = [opt.generate_evaluation_specification() for _ in range(8)]
+        run["inital_evaluations"] = [
+            es.create_evaluation(objectives={"loss": _run_experiment_1d(es)})
+            for es in eval_specs
+        ]
+        random.seed(run_idx)
+        random.shuffle(run["inital_evaluations"])
+        opt.report(run["inital_evaluations"])
+
+        # Start optimizing
+        for _ in range(5):
+            es = opt.generate_evaluation_specification()
+            evaluation = es.create_evaluation(
+                objectives={"loss": _run_experiment_1d(es)}
+            )
+            opt.report(evaluation)
+            run["evaluations"].append(evaluation)
+
+    initial_configs_run_0 = [e.configuration for e in runs[0]["inital_evaluations"]]
+    initial_configs_run_1 = [e.configuration for e in runs[1]["inital_evaluations"]]
+
+    configs_run_0 = [e.configuration for e in runs[0]["evaluations"]]
+    configs_run_1 = [e.configuration for e in runs[1]["evaluations"]]
+
+    assert initial_configs_run_0 != initial_configs_run_1
+    assert configs_run_0 == configs_run_1
 
 
 def handles_reporting_evaluations_list(
@@ -153,7 +231,7 @@ def handles_reporting_evaluations_list(
         Type[SingleObjectiveOptimizer], Type[MultiObjectiveOptimizer]
     ],
     optimizer_kwargs: dict,
-) -> bool:
+):
     """Check if optimizer's report method can process an iterable of evaluations.
 
     All optimizers should be able to allow reporting batches of evaluations. It's up to
@@ -175,6 +253,7 @@ def handles_reporting_evaluations_list(
         optimizer_kwargs,
         objective=Objective("loss", False),
         objectives=[Objective("loss", False)],
+        seed=42,
     )
     evaluations = []
     for _ in range(3):
@@ -183,7 +262,6 @@ def handles_reporting_evaluations_list(
         evaluations.append(evaluation)
 
     opt.report(evaluations)
-    return True
 
 
 def raises_evaluation_error_when_reporting_unknown_objective(
@@ -191,7 +269,7 @@ def raises_evaluation_error_when_reporting_unknown_objective(
         Type[SingleObjectiveOptimizer], Type[MultiObjectiveOptimizer]
     ],
     optimizer_kwargs: dict,
-) -> bool:
+):
     """Check if optimizer's report method raises exception in case objective is unknown.
 
     Also make sure that the faulty evaluations (and only those) are included in the
@@ -234,15 +312,13 @@ def raises_evaluation_error_when_reporting_unknown_objective(
         assert len(invalid_evaluations) == 1
         assert evaluation_2 in invalid_evaluations
 
-    return True
-
 
 def respects_fixed_parameter(
     optimizer_class: Union[
         Type[SingleObjectiveOptimizer], Type[MultiObjectiveOptimizer]
     ],
     optimizer_kwargs: dict,
-) -> bool:
+):
     """Check if optimizer's generated evaluation specifications contain the values
     a parameter in the search space was fixed to.
 
@@ -275,15 +351,13 @@ def respects_fixed_parameter(
             es.create_evaluation(objectives={"loss": es.configuration["x"] ** 2})
         )
 
-    return True
-
 
 def handles_conditional_space(
     optimizer_class: Union[
         Type[SingleObjectiveOptimizer], Type[MultiObjectiveOptimizer]
     ],
     optimizer_kwargs: dict,
-) -> bool:
+):
     """Check if optimizer handles conditional i.e. hierarchical search spaces.
 
     Args:
@@ -316,12 +390,11 @@ def handles_conditional_space(
         dummy_loss = es.configuration.get("momentum", 1.0) * es.configuration["lr"] ** 2
         opt.report(es.create_evaluation({"loss": dummy_loss}))
 
-    return True
-
 
 ALL_REFERENCE_TESTS = [
     optimize_single_parameter_sequentially_for_n_max_evaluations,
-    is_deterministic_with_fixed_seed,
+    is_deterministic_with_fixed_seed_and_larger_space,
+    is_deterministic_when_reporting_shuffled_evaluations,
     handles_reporting_evaluations_list,
     raises_evaluation_error_when_reporting_unknown_objective,
     respects_fixed_parameter,
